@@ -35,7 +35,7 @@ _CITATION = """\
 _DESCRIPTION = """\
 LibriSpeech is a corpus of approximately 1000 hours of read English speech with sampling rate of 16 kHz,
 prepared by Vassil Panayotov with the assistance of Daniel Povey. The data is derived from read
-audiobooks from the LibriVox project, and has been carefully segmented and aligned.87
+audiobooks from the LibriVox project, and has been carefully segmented and aligned.
 """
 
 _URL = "http://www.openslr.org/12"
@@ -54,14 +54,17 @@ _DL_URLS = {
 class Librispeech(tfds.core.BeamBasedBuilder):
   """Librispeech dataset."""
 
-  VERSION = tfds.core.Version("2.1.0")
+  VERSION = tfds.core.Version("2.1.1")
+  RELEASE_NOTES = {
+      "2.1.1": "Fix speech data type with dtype=tf.int16.",
+  }
 
   def _info(self):
     return tfds.core.DatasetInfo(
         builder=self,
         description=_DESCRIPTION,
         features=tfds.features.FeaturesDict({
-            "speech": tfds.features.Audio(sample_rate=16000),
+            "speech": tfds.features.Audio(sample_rate=16000, dtype=tf.int16),
             "text": tfds.features.Text(),
             "speaker_id": tf.int64,
             "chapter_id": tf.int64,
