@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 
 import collections
 import os
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 _CITATION = """
@@ -44,7 +44,9 @@ NUM_CLASSES = 1252
 class Voxceleb(tfds.core.GeneratorBasedBuilder):
   """The VoxCeleb dataset for speaker identification."""
 
-  VERSION = tfds.core.Version('1.1.1')
+  VERSION = tfds.core.Version('1.2.1')
+
+  RELEASE_NOTES = {'1.2.1': 'Add youtube_id field'}
 
   MANUAL_DOWNLOAD_INSTRUCTIONS = """
   manual_dir should contain the file vox_dev_wav.zip. The instructions for
