@@ -15,7 +15,8 @@
 
 """FeatureDict: Main feature connector container."""
 
-from typing import Dict, Union
+import concurrent.futures
+from typing import Dict, List, Union
 
 import tensorflow as tf
 
@@ -28,6 +29,7 @@ from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import type_utils
 
 Json = type_utils.Json
+WORKER_COUNT = 16
 
 
 class _DictGetCounter(object):
@@ -110,7 +112,12 @@ class FeaturesDict(top_level_feature.TopLevelFeature):
 
   """
 
-  def __init__(self, feature_dict: Dict[str, feature_lib.FeatureConnectorArg]):
+  def __init__(
+      self,
+      feature_dict: Dict[str, feature_lib.FeatureConnectorArg],
+      *,
+      doc: feature_lib.DocArg = None,
+  ):
     """Initialize the features.
 
     Args:
@@ -118,11 +125,12 @@ class FeaturesDict(top_level_feature.TopLevelFeature):
         example. The keys should correspond to the data dict as returned by
         tf.data.Dataset(). Types (tf.int32,...) and dicts will automatically be
         converted into FeatureConnector.
+      doc: Documentation of this feature (e.g. description).
 
     Raises:
       ValueError: If one of the given features is not recognized
     """
-    super(FeaturesDict, self).__init__()
+    super(FeaturesDict, self).__init__(doc=doc)
     self._feature_dict = {k: to_feature(v) for k, v in feature_dict.items()}
 
   # Dict functions.
@@ -163,6 +171,26 @@ class FeaturesDict(top_level_feature.TopLevelFeature):
     lines.append('})')
     return '\n'.join(lines)
 
+  def catalog_documentation(
+      self) -> List[feature_lib.CatalogFeatureDocumentation]:
+    feature_docs = [
+        feature_lib.CatalogFeatureDocumentation(
+            name='',
+            cls_name=type(self).__name__,
+            tensor_info=None,
+            description=self._doc.desc,
+            value_range=self._doc.value_range,
+        )
+    ]
+    for feature_name, feature in sorted(list(self._feature_dict.items())):
+      for documentation in feature.catalog_documentation():
+        if documentation.name:
+          nested_name = f'{feature_name}/{documentation.name}'
+        else:
+          nested_name = feature_name
+        feature_docs.append(documentation.replace(name=nested_name))
+    return feature_docs
+
   @py_utils.memoize()
   def get_tensor_info(self):
     """See base class for details."""
@@ -251,20 +279,24 @@ class FeaturesDict(top_level_feature.TopLevelFeature):
   def save_metadata(self, data_dir, feature_name=None):
     """See base class for details."""
     # Recursively save all child features
-    for feature_key, feature in self._feature_dict.items():
-      feature_key = feature_key.replace('/', '.')
-      if feature_name:
-        feature_key = '-'.join((feature_name, feature_key))
-      feature.save_metadata(data_dir, feature_name=feature_key)
+    for child_name, feature in self._feature_dict.items():
+      name_for_file = feature_lib.convert_feature_name_to_filename(
+          feature_name=child_name, parent_name=feature_name)
+      feature.save_metadata(data_dir, feature_name=name_for_file)
 
   def load_metadata(self, data_dir, feature_name=None):
     """See base class for details."""
-    # Recursively load all child features
-    for feature_key, feature in self._feature_dict.items():
-      feature_key = feature_key.replace('/', '.')
-      if feature_name:
-        feature_key = '-'.join((feature_name, feature_key))
-      feature.load_metadata(data_dir, feature_name=feature_key)
+
+    # Load all child features asynchronously
+    def load_metadata(feature_item):
+      child_name, feature = feature_item
+      name_for_file = feature_lib.convert_feature_name_to_filename(
+          feature_name=child_name, parent_name=feature_name)
+      feature.load_metadata(data_dir, feature_name=name_for_file)
+
+    with concurrent.futures.ThreadPoolExecutor(
+        max_workers=WORKER_COUNT) as executor:
+      executor.map(load_metadata, self._feature_dict.items())
 
 
 def to_feature(value: feature_lib.FeatureConnectorArg):
