@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@
 from typing import Optional
 
 from absl import logging
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import dataset_info
 from tensorflow_datasets.core import dataset_utils
@@ -95,6 +95,7 @@ class ImageGridVisualizer(visualizer.Visualizer):
       cols: int = 3,
       plot_scale: float = 3.,
       image_key: Optional[str] = None,
+      label_key: Optional[str] = None,
   ):
     """Display the dataset.
 
@@ -110,6 +111,8 @@ class ImageGridVisualizer(visualizer.Visualizer):
         get overlapped.
       image_key: `string`, name of the feature that contains the image. If not
         set, the system will try to auto-detect it.
+      label_key: `string`, name of the feature that contains the label. If not
+        set, the system will try to auto-detect it.
 
     Returns:
       fig: The pyplot figure.
@@ -127,9 +130,13 @@ class ImageGridVisualizer(visualizer.Visualizer):
     # Optionally extract the label key
     label_keys = visualizer.extract_keys(ds_info.features,
                                          features_lib.ClassLabel)
-    label_key = label_keys[0] if len(label_keys) == 1 else None
-    if not label_key:
-      logging.info('Was not able to auto-infer label.')
+    if label_key is not None:
+      assert label_key in label_keys, (
+          f'Label "{label_key}" not found: {label_keys}.')
+    else:
+      label_key = label_keys[0] if len(label_keys) == 1 else None
+      if not label_key:
+        logging.info('Was not able to auto-infer label.')
 
     # Single image display
     def make_cell_fn(ax, ex):
