@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 # pylint: disable=g-import-not-at-top,g-importing-member, import-outside-toplevel
 
 try:  # pylint: disable=g-statement-before-imports
-  from waymo_open_dataset import waymo_dataset_pb2
+  from waymo_open_dataset import dataset_pb2 as waymo_dataset_pb2
 except ImportError:
   # If original waymo proto is not found, fallback to the pre-generated proto
   from tensorflow_datasets.proto import waymo_dataset_generated_pb2 as waymo_dataset_pb2  # pylint: disable=line-too-long
