@@ -18,15 +18,17 @@
 import argparse
 import functools
 import importlib
+import json
 import os
 import pathlib
 import typing
-from typing import Dict, Iterator, Optional, Tuple, Type
+from typing import Dict, Iterator, Optional, Tuple, Type, Union
 
 from absl import logging
+from etils import epath
 import tensorflow_datasets as tfds
 
-# pylint: disable=logging-format-interpolation
+# pylint: disable=logging-fstring-interpolation
 
 
 def register_subparser(parsers: argparse._SubParsersAction) -> None:  # pylint: disable=protected-access
@@ -58,6 +60,12 @@ def register_subparser(parsers: argparse._SubParsersAction) -> None:  # pylint:
       action='store_true',
       help='Delete pre-existing dataset if it exists.',
   )
+  debug_group.add_argument(
+      '--fail_if_exists',
+      action='store_true',
+      default=False,
+      help='Fails the program if there is a pre-existing dataset.',
+  )
   debug_group.add_argument(
       '--max_examples_per_split',
       type=int,
@@ -73,27 +81,28 @@ def register_subparser(parsers: argparse._SubParsersAction) -> None:  # pylint:
   path_group = build_parser.add_argument_group('Paths')
   path_group.add_argument(
       '--data_dir',
-      type=tfds.core.as_path,
+      type=tfds.core.Path,
       # Should match tfds.core.constant.DATA_DIR !!
-      default=tfds.core.as_path(
-          os.environ.get('TFDS_DATA_DIR',
-                         os.path.join('~', 'tensorflow_datasets'))),
+      default=tfds.core.Path(
+          os.environ.get(
+              'TFDS_DATA_DIR',
+              os.path.join(os.path.expanduser('~'), 'tensorflow_datasets'))),
       help='Where to place datasets. Default to '
       '`~/tensorflow_datasets/` or `TFDS_DATA_DIR` environement variable.',
   )
   path_group.add_argument(
       '--download_dir',
-      type=tfds.core.as_path,
+      type=tfds.core.Path,
       help='Where to place downloads. Default to `<data_dir>/downloads/`.',
   )
   path_group.add_argument(
       '--extract_dir',
-      type=tfds.core.as_path,
+      type=tfds.core.Path,
       help='Where to extract files. Default to `<download_dir>/extracted/`.',
   )
   path_group.add_argument(
       '--manual_dir',
-      type=tfds.core.as_path,
+      type=tfds.core.Path,
       help='Where to manually download data (required for some datasets). '
       'Default to `<download_dir>/manual/`.',
   )
@@ -110,7 +119,9 @@ def register_subparser(parsers: argparse._SubParsersAction) -> None:  # pylint:
       '--config',
       '-c',
       type=str,
-      help='Config name to build. Build all configs if not set.')
+      help='Config name to build. Build all configs if not set. '
+      'Can also be a json of the kwargs forwarded to the config `__init__` ('
+      'for custom configs).')
   # We are forced to have 2 flags to avoid ambiguity when config name is
   # a number (e.g. `voc/2017`)
   generation_group.add_argument(
@@ -146,8 +157,29 @@ def register_subparser(parsers: argparse._SubParsersAction) -> None:  # pylint:
   generation_group.add_argument(
       '--file_format',
       type=str,
-      help='File format to which generate the tf-examples. '
-      f'Available values: {format_values} (see `tfds.core.FileFormat`).',
+      help=('File format to which generate the tf-examples. '
+            f'Available values: {format_values} (see `tfds.core.FileFormat`).'),
+  )
+
+  publish_group = build_parser.add_argument_group(
+      'Publishing',
+      description='Options for publishing successfully created datasets.')
+  publish_group.add_argument(
+      '--publish_dir',
+      type=tfds.core.Path,
+      default=None,
+      required=False,
+      help=('Where to optionally publish the dataset after it has been '
+            'generated successfully. Should be the root data dir under which'
+            'datasets are stored. '
+            'If unspecified, dataset will not be published'),
+  )
+  publish_group.add_argument(
+      '--skip_if_published',
+      action='store_true',
+      default=False,
+      help=('If the dataset with the same version and config is already '
+            'published, then it will not be regenerated.'),
   )
 
   # **** Automation options ****
@@ -197,7 +229,10 @@ def _make_builders(
     ds_to_build: str,
 ) -> Iterator[tfds.core.DatasetBuilder]:
   """Yields builders to generate."""
-  builder_cls, builder_kwargs = _get_builder_cls(ds_to_build)
+  builder_cls, builder_kwargs = _get_builder_cls(
+      ds_to_build,
+      has_imports=bool(args.imports),
+  )
 
   # Eventually overwrite version
   if args.experimental_latest_version:
@@ -221,6 +256,7 @@ def _make_builders(
       _make_builder,
       builder_cls,
       overwrite=args.overwrite,
+      fail_if_exists=args.fail_if_exists,
       data_dir=args.data_dir,
       **builder_kwargs,
   )
@@ -235,24 +271,31 @@ def _make_builders(
 
 
 def _get_builder_cls(
-    ds_to_build: str,) -> Tuple[Type[tfds.core.DatasetBuilder], Dict[str, str]]:
+    ds_to_build: str,
+    *,
+    has_imports: bool,
+) -> Tuple[Type[tfds.core.DatasetBuilder], Dict[str, str]]:
   """Infer the builder class to build.
 
   Args:
     ds_to_build: Dataset argument.
+    has_imports: Whether `--imports` was passed
 
   Returns:
     builder_cls: The dataset class to download and prepare
     kwargs:
   """
   # 1st case: Requested dataset is a path to `.py` script
-  path = _search_script_path(ds_to_build)
-  if path is not None:
-    logging.info(f'Loading dataset {ds_to_build} from path: {path}')
-    # Dynamically load user dataset script
-    with tfds.core.utils.add_sys_path(path.parent):
-      builder_cls = tfds.core.community.builder_cls_from_module(path.stem)
-    return builder_cls, {}
+  # When `--imports=` is set, it means the user expects the dataset to be
+  # registered through the `imports` and not locally.
+  if not has_imports:
+    path = _search_script_path(ds_to_build)
+    if path is not None:
+      logging.info(f'Loading dataset {ds_to_build} from path: {path}')
+      # Dynamically load user dataset script
+      with tfds.core.utils.add_sys_path(path.parent):
+        builder_cls = tfds.core.community.builder_cls_from_module(path.stem)
+      return builder_cls, {}
 
   # 2nd case: Dataset is registered through imports.
 
@@ -265,7 +308,7 @@ def _get_builder_cls(
   return builder_cls, builder_kwargs
 
 
-def _search_script_path(ds_to_build: str) -> Optional[tfds.core.ReadOnlyPath]:
+def _search_script_path(ds_to_build: str) -> Optional[tfds.core.Path]:
   """Check whether the requested dataset match a file on disk."""
   # If the dataset file exists, use it. Valid values are:
   # * Empty string (use default directory)
@@ -276,9 +319,9 @@ def _search_script_path(ds_to_build: str) -> Optional[tfds.core.ReadOnlyPath]:
   # TODO(py3.7): Should be `path.expanduser().resolve()` but `.resolve()` fails
   # on some environments when the file doesn't exists.
   # https://stackoverflow.com/questions/55710900/pathlib-resolve-method-not-resolving-non-existant-files
-  path = tfds.core.as_path(ds_to_build).expanduser()
+  path = tfds.core.Path(ds_to_build).expanduser()
   if not path.exists():
-    path = tfds.core.as_path(pathlib.Path()).resolve() / path
+    path = tfds.core.Path(pathlib.Path()).resolve() / path
   else:
     path = path.resolve()
 
@@ -294,7 +337,7 @@ def _search_script_path(ds_to_build: str) -> Optional[tfds.core.ReadOnlyPath]:
     return path
   # Path not found. Use heuristic to detect if user intended to pass a path:
   elif (not ds_to_build or ds_to_build.endswith((os.sep, '.py'))  # ds.py
-        or tfds.core.as_path(ds_to_build).is_absolute()  # /path/to
+        or tfds.core.Path(ds_to_build).is_absolute()  # /path/to
         or ds_to_build.count(os.sep) > 1  # path/dataset/config
        ):
     raise FileNotFoundError(
@@ -304,8 +347,7 @@ def _search_script_path(ds_to_build: str) -> Optional[tfds.core.ReadOnlyPath]:
     return None
 
 
-def _validate_script_path(
-    path: tfds.core.ReadOnlyPath,) -> Optional[tfds.core.ReadOnlyPath]:
+def _validate_script_path(path: tfds.core.Path,) -> Optional[tfds.core.Path]:
   """Validates and returns the `dataset.py` generation script path."""
   if path.suffix != '.py':
     raise ValueError(f'Expected `.py` file. Invalid dataset path: {path}.')
@@ -318,11 +360,16 @@ def _validate_script_path(
 def _make_builder(
     builder_cls: Type[tfds.core.DatasetBuilder],
     overwrite: bool,
+    fail_if_exists: bool,
     **builder_kwargs,
 ) -> tfds.core.DatasetBuilder:
   """Builder factory, eventually deleting pre-existing dataset."""
   builder = builder_cls(**builder_kwargs)  # pytype: disable=not-instantiable
-  if overwrite and builder.data_path.exists():
+  data_exists = builder.data_path.exists()
+  if fail_if_exists and data_exists:
+    raise RuntimeError('The `fail_if_exists` flag was True and '
+                       f'the data already exists in {builder.data_path}')
+  if overwrite and data_exists:
     builder.data_path.rmtree()  # Delete pre-existing data
     # Re-create the builder with clean state
     builder = builder_cls(**builder_kwargs)  # pytype: disable=not-instantiable
@@ -336,6 +383,13 @@ def _download_and_prepare(
   """Generate a single builder."""
   logging.info(f'download_and_prepare for dataset {builder.info.full_name}...')
 
+  publish_data_dir = _publish_data_dir(args.publish_dir, builder)
+  if args.skip_if_published and publish_data_dir is not None:
+    if publish_data_dir.exists():
+      logging.info(
+          f'Dataset already exists in {publish_data_dir}. Skipping generation.')
+      return
+
   dl_config = _make_download_config(args)
   if args.add_name_to_manual_dir:
     dl_config.manual_dir = os.path.join(dl_config.manual_dir, builder.name)
@@ -347,11 +401,49 @@ def _download_and_prepare(
 
   # Dataset generated successfully
   logging.info('Dataset generation complete...')
+
   print()
   info_repr = repr(builder.info)
   print(info_repr)
   print()
 
+  # Publish data if requested.
+  if publish_data_dir is not None:
+    logging.info(f'Publishing dataset to {publish_data_dir}')
+    _publish_data(
+        publish_data_dir=publish_data_dir,
+        builder=builder,
+        overwrite=args.overwrite)
+    logging.info(f'Dataset successfully published to {publish_data_dir}')
+
+
+def _publish_data_dir(
+    publish_dir: Optional[tfds.core.Path],
+    builder: tfds.core.DatasetBuilder) -> Optional[epath.Path]:
+  if publish_dir is None:
+    return None
+  return epath.Path(publish_dir) / builder.info.full_name
+
+
+def _publish_data(
+    publish_data_dir: epath.Path,
+    builder: tfds.core.DatasetBuilder,
+    overwrite: bool = False,
+) -> None:
+  """Publishes the data from the given builder to `publish_data_dir`.
+
+  Arguments:
+    publish_data_dir: folder where the data should be published. Should include
+      config and version.
+    builder: the builder whose data needs to be published.
+    overwrite: whether to overwrite existing data in the `publish_root_dir` if
+      it exists.
+  """
+  from_data_dir = epath.Path(builder.data_dir)
+  publish_data_dir.mkdir(parents=True, exist_ok=True)
+  for filepath in from_data_dir.iterdir():
+    filepath.copy(dst=publish_data_dir / filepath.name, overwrite=overwrite)
+
 
 def _make_download_config(
     args: argparse.Namespace,) -> tfds.download.DownloadConfig:
@@ -385,7 +477,7 @@ def _get_config_name(
     config_kwarg: Optional[str],
     config_name: Optional[str],
     config_idx: Optional[int],
-) -> Optional[str]:
+) -> Optional[Union[str, tfds.core.BuilderConfig]]:
   """Extract the config name.
 
   Args:
@@ -411,7 +503,11 @@ def _get_config_name(
   if config_kwarg:  # `ds_name/config`
     return config_kwarg
   elif config_name:  # `--config name`
-    return config_name
+    if config_name.startswith('{'):
+      # Json: Dynamically recreate the builder_config
+      return builder_cls.builder_config_cls(**json.loads(config_name))
+    else:
+      return config_name
   elif config_idx is not None:  # `--config_idx 123`
     if config_idx > len(builder_cls.BUILDER_CONFIGS):
       raise ValueError(f'--config_idx {config_idx} greater than number '
