@@ -21,9 +21,13 @@ import pathlib
 from typing import Iterator, List
 from unittest import mock
 
+from etils import epath
 import pytest
 
 import tensorflow_datasets as tfds
+from tensorflow_datasets import testing
+from tensorflow_datasets.core.utils import file_utils
+from tensorflow_datasets.scripts.cli import build as build_lib
 from tensorflow_datasets.scripts.cli import main
 
 type_utils = tfds.core.utils.type_utils
@@ -55,7 +59,7 @@ def mock_default_data_dir(tmp_path: pathlib.Path):
 
 
 @contextlib.contextmanager
-def mock_cwd(path: type_utils.PathLike) -> Iterator[None]:
+def mock_cwd(path: epath.PathLike) -> Iterator[None]:
   """Mock the current directory."""
   path = pathlib.Path(path)
   assert path.exists() and path.is_dir()  # Check given path is valid cwd dir
@@ -143,6 +147,12 @@ def test_build_dataset_configs():
       'trivia_qa/unfiltered.nocontext',
   ]
 
+  # --config Json
+  config_json = '{"name":"my_config","description":"abcd"}'
+  assert _build(f'imdb_reviews --config {config_json}') == [
+      'imdb_reviews/my_config',
+  ]
+
   # name/config
   assert _build('trivia_qa/unfiltered.nocontext') == [
       'trivia_qa/unfiltered.nocontext'
@@ -210,19 +220,25 @@ def test_build_files():
   with pytest.raises(FileNotFoundError, match='Could not find .* script'):
     _build('')
 
-  # cd .../datasets/dummy_dataset && gtfds build
+  # cd .../datasets/dummy_dataset && tfds build
   with mock_cwd(_DUMMY_DATASET_PATH):
     assert _build('') == ['dummy_dataset']
 
-  # cd .../datasets/dummy_dataset && gtfds build dummy_dataset.py
+  # cd .../datasets/dummy_dataset && tfds build dummy_dataset.py
   with mock_cwd(_DUMMY_DATASET_PATH):
     assert _build('dummy_dataset.py') == ['dummy_dataset']
 
-  # cd .../datasets/ && gtfds build dummy_dataset
+  # cd .../datasets/ && tfds build dummy_dataset
   with mock_cwd(_DUMMY_DATASET_PATH.parent):
     assert _build('dummy_dataset') == ['dummy_dataset']
 
-  # cd .../datasets/ && gtfds build dummy_dataset/dummy_dataset
+  # cd .../datasets/ && tfds build dummy_dataset --imports=xxx
+  # --imports is passed. so do not load dataset from file
+  with mock_cwd(_DUMMY_DATASET_PATH.parent):
+    with pytest.raises(tfds.core.registered.DatasetNotFoundError):
+      assert _build('dummy_dataset --imports=os')
+
+  # cd .../datasets/ && tfds build dummy_dataset/dummy_dataset
   with mock_cwd(_DUMMY_DATASET_PATH.parent):
     assert _build('dummy_dataset/dummy_dataset') == ['dummy_dataset']
 
@@ -239,3 +255,23 @@ def test_build_import():
   # --imports register the dataset
   ds_module = 'tensorflow_datasets.testing.dummy_dataset.dummy_dataset'
   assert _build(f'dummy_dataset --imports {ds_module}') == ['dummy_dataset']
+
+
+def test_publish_data_dir(mock_fs: testing.MockFs):
+  del mock_fs
+  builder = testing.DummyMnist(data_dir='/tmp')
+  actual = build_lib._publish_data_dir(
+      publish_dir=epath.Path('/a/b'), builder=builder)
+  assert actual == epath.Path('/a/b/dummy_mnist/3.0.1')
+  assert build_lib._publish_data_dir(publish_dir=None, builder=builder) is None
+
+
+def test_publish_data(mock_fs: testing.MockFs):
+  builder = testing.DummyMnist(data_dir='/tmp')
+  expected_from = epath.Path('/tmp') / 'dummy_mnist/3.0.1'
+  filename = 'dataset_info.json'
+  content = 'a'
+  mock_fs.add_file(path=expected_from / filename, content=content)
+  publish_data_dir = epath.Path('/a/b')
+  build_lib._publish_data(publish_data_dir=publish_data_dir, builder=builder)
+  assert mock_fs.read_file(publish_data_dir / filename) == content
