@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,6 @@ See: https://www.tensorflow.org/datasets/cli
 
 import argparse
 import logging as python_logging
-import re
 import sys
 from typing import List
 
@@ -38,18 +37,19 @@ import tensorflow_datasets.public_api as tfds
 # Import commands
 from tensorflow_datasets.scripts.cli import build
 from tensorflow_datasets.scripts.cli import new
+from tensorflow_datasets.scripts.utils import flag_utils
 
 FLAGS = flags.FLAGS
 
 
 def _parse_flags(argv: List[str]) -> argparse.Namespace:
   """Command lines flag parsing."""
-  # Normalize explicit boolean flags for absl.flags compatibility
-  # See b/174043007 for context.
-  argv = _normalize_flags(argv)
+  argv = flag_utils.normalize_flags(argv)  # See b/174043007 for context.
 
   parser = argparse_flags.ArgumentParser(
-      description='Tensorflow Datasets CLI tool',)
+      description='Tensorflow Datasets CLI tool',
+      allow_abbrev=False,
+  )
   parser.add_argument(
       '--version',
       action='version',
@@ -62,24 +62,6 @@ def _parse_flags(argv: List[str]) -> argparse.Namespace:
   return parser.parse_args(argv[1:])
 
 
-def _normalize_flags(argv: List[str]) -> List[str]:
-  """Normalize explicit bolean flags for absl.flags compatibility."""
-  bolean_flag_patern = re.compile(r'--[\w_]+=(true|false)')
-
-  def _normalize_flag(arg: str) -> str:
-    if not bolean_flag_patern.match(arg):
-      return arg
-    if arg.endswith('=true'):
-      return arg[:-len('=true')]  # `--flag=true` -> `--flag`
-    elif arg.endswith('=false'):
-      # `--flag=false` -> `--noflag`
-      return '--no' + arg[len('--'):-len('=false')]
-    else:
-      raise AssertionError(f'Unrecognized arg: {arg}')
-
-  return [_normalize_flag(a) for a in argv]
-
-
 def main(args: argparse.Namespace) -> None:
 
   # From the CLI, all datasets are visible
