@@ -28,6 +28,7 @@ from typing import Any, Dict, Iterator, List, Optional, Type
 from absl import logging
 import tensorflow as tf
 import tensorflow_datasets as tfds
+from tensorflow_datasets.scripts.documentation import collection_markdown_builder
 from tensorflow_datasets.scripts.documentation import dataset_markdown_builder
 from tensorflow_datasets.scripts.documentation import doc_utils
 import tqdm
@@ -74,6 +75,20 @@ class BuilderDocumentation:
   is_nightly: bool
 
 
+@dataclasses.dataclass(eq=False, frozen=True)
+class CollectionDocumentation:
+  """Documentation output of a single builder.
+
+  Attributes:
+    name: Documentation page name (e.g. `xtreme`)
+    content: Documentation content
+    is_nightly: Whether the dataset was recently added in `tfds-nightly`
+  """
+  name: str
+  content: str
+  is_nightly: bool
+
+
 def _load_builder(name: str,) -> Optional[BuilderToDocument]:
   """Load the builder to document.
 
@@ -85,7 +100,7 @@ def _load_builder(name: str,) -> Optional[BuilderToDocument]:
     builder: Main builder instance
     config_builders: Additional builders (one of each configs)
   """
-  if tfds.core.utils.DatasetName(name).namespace:  # Community dataset
+  if tfds.core.naming.DatasetName(name).namespace:  # Community dataset
     return _load_builder_from_location(name)
   else:  # Code dataset
     return _load_builder_from_code(name)
@@ -93,7 +108,7 @@ def _load_builder(name: str,) -> Optional[BuilderToDocument]:
 
 def _load_builder_from_location(name: str,) -> Optional[BuilderToDocument]:
   """Load the builder, config,... to document."""
-  dataset_name = tfds.core.utils.DatasetName(name)
+  dataset_name = tfds.core.naming.DatasetName(name)
   logging.info(f'Loading builder {dataset_name} from location')
   try:
     builder = tfds.builder(name)
@@ -108,7 +123,7 @@ def _load_builder_from_location(name: str,) -> Optional[BuilderToDocument]:
     if not builder:
       logging.error(f'Dataset {dataset_name} not found', exc_info=e)
       return None
-  except tf.errors.PermissionDeniedError as e:
+  except (OSError, tf.errors.PermissionDeniedError) as e:
     logging.error(f'Permission denied for {dataset_name}', exc_info=e)
     tqdm.tqdm.write(f'Warning: Skip dataset {name} due to permission error')
     return None
@@ -150,7 +165,7 @@ def _load_all_configs(
       return None
     if not builder_conf.builder_config:
       # Unexpected sub-config with wrong metadata.
-      # This can happen if the user manually mess-up with the directories.
+      # This can happen if the user manually messed up with the directories.
       return None
     return builder_conf
 
@@ -202,6 +217,20 @@ def _get_section(builder_cls: Type[tfds.core.DatasetBuilder]) -> str:
   return category
 
 
+def _document_single_collection(name: str,) -> CollectionDocumentation:
+  """Returns the documentation for a single dataset collection."""
+  logging.info('Documenting dataset collection %s', name)
+  collection = tfds.dataset_collection(name=name)
+  out_str = collection_markdown_builder.get_collection_markdown_string(
+      collection=collection)
+  return CollectionDocumentation(
+      name=name,
+      content=out_str,
+      # TODO(tfds): Add support for display of nightly icon when necessary.
+      is_nightly=False,
+  )
+
+
 def _document_single_builder(
     name: str,
     **kwargs: Any,
@@ -250,12 +279,22 @@ def _all_tfds_datasets() -> List[str]:
   ])
 
 
+def iter_collections_documentation(
+    collection_names: Optional[List[str]] = None,
+) -> Iterator[CollectionDocumentation]:
+  logging.info('Retrieving dataset collection names...')
+  collection_names = collection_names or sorted(tfds.list_dataset_collections())
+  for collection_name in collection_names:
+    yield _document_single_collection(name=collection_name)
+  print('All collections documentations generated!')
+
+
 def iter_documentation_builders(
     datasets: Optional[List[str]] = None,
     *,
     doc_util_paths: Optional[doc_utils.DocUtilPaths] = None,
 ) -> Iterator[BuilderDocumentation]:
-  """Create dataset documentation string for given datasets.
+  """Creates dataset documentation string for given datasets.
 
   Args:
     datasets: list of datasets for which to create documentation. If None, then
