@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Base DatasetBuilderTestCase to test a DatasetBuilder base class."""
+"""Base DatasetBuilderTestCase to test a DatasetBuilder."""
 
 import collections
 import contextlib
@@ -23,11 +23,12 @@ import itertools
 import numbers
 import os
 import textwrap
-from typing import Iterator, List, Optional, Union
+from typing import Iterator, List, Mapping, Optional, Union
 from unittest import mock
 
 from absl import logging
 from absl.testing import parameterized
+from etils import epath
 import numpy as np
 import tensorflow as tf
 
@@ -91,6 +92,7 @@ class DatasetBuilderTestCase(parameterized.TestCase,
   You must set the following class attributes:
 
     * DATASET_CLASS: class object of DatasetBuilder you want to test.
+    * SPLITS: dict mapping split name to expected number of examples.
 
   You may set the following class attributes:
 
@@ -139,6 +141,7 @@ class DatasetBuilderTestCase(parameterized.TestCase,
   """
 
   DATASET_CLASS = None
+  SPLITS = None
   VERSION = None
   BUILDER_CONFIG_NAMES_TO_TEST: Optional[List[Union[
       str, dataset_builder.BuilderConfig]]] = None
@@ -149,7 +152,7 @@ class DatasetBuilderTestCase(parameterized.TestCase,
   OVERLAPPING_SPLITS = []
   MOCK_OUT_FORBIDDEN_OS_FUNCTIONS = True
   SKIP_CHECKSUMS = False
-  SKIP_TF1_GRAPH_MODE = False
+  SKIP_TF1_GRAPH_MODE = True
 
   @classmethod
   def setUpClass(cls):
@@ -195,15 +198,15 @@ class DatasetBuilderTestCase(parameterized.TestCase,
   @utils.classproperty
   @classmethod
   @utils.memoize()
-  def dummy_data(cls) -> utils.ReadOnlyPath:  # pylint: disable=no-self-argument
+  def dummy_data(cls) -> epath.Path:  # pylint: disable=no-self-argument
     """Path to the `dummy_data/` directory."""
     if cls is DatasetBuilderTestCase:  # Required for build_api_docs
       return None  # pytype: disable=bad-return-type
 
     dummy_data_expected = cls.DATASET_CLASS.code_path.parent / "dummy_data"
-    fake_example_dir = utils.as_path(test_utils.fake_examples_dir())
+    fake_example_dir = epath.Path(test_utils.fake_examples_dir())
     if cls.EXAMPLE_DIR is not None:
-      dummy_data_found = utils.as_path(cls.EXAMPLE_DIR)
+      dummy_data_found = epath.Path(cls.EXAMPLE_DIR)
       dummy_data_expected = dummy_data_found  # Dir to display in the error
     elif dummy_data_expected.exists():
       dummy_data_found = dummy_data_expected
@@ -272,23 +275,26 @@ class DatasetBuilderTestCase(parameterized.TestCase,
     else:
       self._download_urls.add(url_or_urls)
 
+  def _prepare_download_results(
+      self,
+      downloaded_files: Mapping[str, str],
+  ) -> Mapping[str, epath.PathLike]:
+    return tf.nest.map_structure(
+        lambda fname: self.dummy_data / fname,
+        downloaded_files,
+    )
+
   def _get_dl_extract_result(self, url):
     tf.nest.map_structure(self._add_url, url)
     del url
     if self.DL_EXTRACT_RESULT is None:
       return self.dummy_data
-    return tf.nest.map_structure(
-        lambda fname: self.dummy_data / fname,
-        self.DL_EXTRACT_RESULT,
-    )
+    return self._prepare_download_results(self.DL_EXTRACT_RESULT)
 
   def _get_dl_extract_only_result(self, url):
     if self.DL_EXTRACT_ONLY_RESULT:
       tf.nest.map_structure(self._add_url, url)
-      return tf.nest.map_structure(
-          lambda fname: self.dummy_data / fname,
-          self.DL_EXTRACT_ONLY_RESULT,
-      )
+      return self._prepare_download_results(self.DL_EXTRACT_ONLY_RESULT)
 
   def _get_dl_download_result(self, url):
     tf.nest.map_structure(self._add_url, url)
@@ -296,10 +302,10 @@ class DatasetBuilderTestCase(parameterized.TestCase,
       # This is only to be backwards compatible with old approach.
       # In the future it will be replaced with using self.dummy_data.
       return self._get_dl_extract_result(url)
-    return tf.nest.map_structure(
-        lambda fname: self.dummy_data / fname,
-        self.DL_DOWNLOAD_RESULT,
-    )
+    return self._prepare_download_results(self.DL_DOWNLOAD_RESULT)
+
+  def _get_dl_download_kaggle_result(self, competition_or_dataset):
+    return self._prepare_download_results(self.DL_DOWNLOAD_RESULT)
 
   def _download_checksums(self, url):
     self._stop_record_download = True
@@ -355,7 +361,7 @@ class DatasetBuilderTestCase(parameterized.TestCase,
     err_msg = (
         "Did you forget to record checksums with `--register_checksums` ? See "
         "instructions at: "
-        "hhttps://www.tensorflow.org/datasets/add_dataset#run_the_generation_codeIf"
+        "https://www.tensorflow.org/datasets/add_dataset#run_the_generation_codeIf"
         " want to opt-out of checksums validation, please add `SKIP_CHECKSUMS "
         "= True` to the `DatasetBuilderTestCase`.\n")
     url_infos = self.DATASET_CLASS.url_infos
@@ -392,6 +398,7 @@ class DatasetBuilderTestCase(parameterized.TestCase,
     patches = {
         "download_and_extract": self._get_dl_extract_result,
         "download": self._get_dl_download_result,
+        "download_kaggle_data": self._get_dl_download_kaggle_result,
         "download_checksums": self._download_checksums,
         "manual_dir": manual_dir,
         "download_dir": self.dummy_data
@@ -430,6 +437,9 @@ class DatasetBuilderTestCase(parameterized.TestCase,
     with self._subTest("config_description"):
       self._test_description_builder_config(builder)
 
+    with self._subTest("default_config"):
+      self._test_default_builder_config(builder)
+
   @contextlib.contextmanager
   def _test_key_not_local_path(self, builder) -> Iterator[None]:
     if not isinstance(builder, dataset_builder.GeneratorBasedBuilder):
@@ -527,6 +537,12 @@ class DatasetBuilderTestCase(parameterized.TestCase,
       if ratio > 0.9:
         raise AssertionError(err_msg)
 
+  def _test_default_builder_config(self, builder):
+    if not builder.DEFAULT_BUILDER_CONFIG_NAME:
+      return
+    builder_config_names = {config.name for config in builder.BUILDER_CONFIGS}
+    self.assertIn(builder.DEFAULT_BUILDER_CONFIG_NAME, builder_config_names)
+
 
 def checksum(example):
   """Computes the md5 for a given example."""
