@@ -18,6 +18,7 @@
 from tensorflow_datasets.text.ag_news_subset import AGNewsSubset
 from tensorflow_datasets.text.anli import Anli
 from tensorflow_datasets.text.assin2 import Assin2
+from tensorflow_datasets.text.beir import Beir
 from tensorflow_datasets.text.blimp import Blimp
 from tensorflow_datasets.text.bool_q import BoolQ
 from tensorflow_datasets.text.c4 import C4
@@ -25,6 +26,7 @@ from tensorflow_datasets.text.cfq import CFQ
 from tensorflow_datasets.text.cfq import CFQConfig
 from tensorflow_datasets.text.civil_comments import CivilComments
 from tensorflow_datasets.text.clinc_oos import ClincOOS
+from tensorflow_datasets.text.conll2003 import Conll2003
 from tensorflow_datasets.text.cos_e import CosE
 from tensorflow_datasets.text.definite_pronoun_resolution import DefinitePronounResolution
 from tensorflow_datasets.text.docnli import DocNLI
@@ -47,6 +49,7 @@ from tensorflow_datasets.text.lm1b import Lm1b
 from tensorflow_datasets.text.math_dataset import MathDataset
 from tensorflow_datasets.text.math_qa import MathQa
 from tensorflow_datasets.text.movie_rationales import MovieRationales
+from tensorflow_datasets.text.mrqa import MRQA
 from tensorflow_datasets.text.multi_nli import MultiNLI
 from tensorflow_datasets.text.multi_nli_mismatch import MultiNLIMismatch
 from tensorflow_datasets.text.openbookqa import Openbookqa
@@ -65,6 +68,7 @@ from tensorflow_datasets.text.scan import Scan
 from tensorflow_datasets.text.scan import ScanConfig
 from tensorflow_datasets.text.schema_guided_dialogue import SchemaGuidedDialogue
 from tensorflow_datasets.text.scicite import Scicite
+from tensorflow_datasets.text.scitail import SciTail
 from tensorflow_datasets.text.scrolls import Scrolls
 from tensorflow_datasets.text.sentiment140 import Sentiment140
 from tensorflow_datasets.text.snli import Snli
@@ -74,7 +78,11 @@ from tensorflow_datasets.text.story_cloze import StoryCloze
 from tensorflow_datasets.text.super_glue import SuperGlue
 from tensorflow_datasets.text.tiny_shakespeare import TinyShakespeare
 from tensorflow_datasets.text.trec import Trec
+from tensorflow_datasets.text.unifiedqa import UnifiedQA
+from tensorflow_datasets.text.universal_dependencies import UniversalDependencies
+from tensorflow_datasets.text.userlibri_lm_data import UserLibriText
 from tensorflow_datasets.text.wiki40b import Wiki40b
+from tensorflow_datasets.text.wiki_dialog import WikiDialog
 from tensorflow_datasets.text.wikiann import Wikiann
 from tensorflow_datasets.text.wikipedia import Wikipedia
 from tensorflow_datasets.text.wikipedia_toxicity_subtypes import WikipediaToxicitySubtypes
@@ -83,5 +91,6 @@ from tensorflow_datasets.text.wordnet import Wordnet
 from tensorflow_datasets.text.wsc273 import Wsc273
 from tensorflow_datasets.text.xnli import Xnli
 from tensorflow_datasets.text.xtreme_pawsx import XtremePawsx
+from tensorflow_datasets.text.xtreme_pos import XtremePos
 from tensorflow_datasets.text.xtreme_xnli import XtremeXnli
 from tensorflow_datasets.text.yelp_polarity import YelpPolarityReviews
