@@ -19,6 +19,7 @@ import collections
 import functools
 import json
 import os
+from typing import Optional, Sequence
 import uuid
 
 from absl import logging
@@ -26,6 +27,8 @@ import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 from tensorflow_datasets.text import c4_utils
 
+PageFeatures = c4_utils.PageFeatures
+
 _DESCRIPTION = """\
 A colossal, cleaned version of Common Crawl's web crawl corpus.
 
@@ -48,7 +51,7 @@ _CITATION = """
   eprint = {1910.10683},
 }
 """
-_VERSION = tfds.core.Version("3.0.1")
+_VERSION = tfds.core.Version("3.1.0")
 
 # TODO(adarob): Remove supported versions. Starting with 3.0.0, all generated
 # datasets are automatically forward compatible. For example,
@@ -60,6 +63,15 @@ _SUPPORTED_VERSIONS = [
     tfds.core.Version("2.2.0"),
 ]
 RELEASE_NOTES = {
+    "3.1.0":
+        "All: Select newest timestamp when deduping by URL (vs random); escape "
+        "badword regexes."
+        "Multilingual: Expand complete list of CC dumps (as of Aug 17, 2022); "
+        "use soft badword filtering (99.9%); do not require spaces for "
+        "Japanese when matching bad words (similar to Thai and Chinese); "
+        "up minimum language detection threshold to 0.95 (from 0.7); allow "
+        "badwords that filter >10% of documents for a given non-spaced "
+        "language due to being common subwords.",
     "3.0.1": "Remove mC4 languages with less than 10k pages.",
     "3.0.0": "Add multilingual version (mC4). Deterministic URL deduplication.",
     "2.3.1": "Hashing change.",
@@ -67,10 +79,9 @@ RELEASE_NOTES = {
     "2.2.1": "Update dataset_info.json",
 }
 
-_DOWNLOAD_HOST = "https://commoncrawl.s3.amazonaws.com"
-_WET_PATH_URL = "https://commoncrawl.s3.amazonaws.com/crawl-data/CC-MAIN-{cc_version}/wet.paths.gz"
+_DOWNLOAD_HOST = "https://data.commoncrawl.org"
+_WET_PATH_URL = "https://data.commoncrawl.org/crawl-data/CC-MAIN-{cc_version}/wet.paths.gz"
 _REALNEWS_DOMAINS_URL = "https://raw.githubusercontent.com/rowanz/grover/38f7184bd87237ae2d3bc330b99f1e2e246f6d51/realnews/domain_to_allowed_subdomains.json"
-_CHECKSUMS_URL = "https://storage.googleapis.com/tfds-data/manual_checksums/c4.txt"
 _OPENWEBTEXT_URLS_ZIP = "OpenWebText.zip"
 _OPENWEBTEXT_URLS_URL = "https://mega.nz/#F!EZZD0YwJ!9_PlEQzdMVLaNdKv_ICNVQ"
 _OPENWEBTEXT_URLS_FILE_PATTERN = "OpenWebText/Version 1/URLs/*.txt"
@@ -81,6 +92,9 @@ _BADWORDS_LANGS = [
     "fr-CA-u-sd-caqc", "hi", "hu", "it", "ja", "kab", "ko", "nl", "no", "pl",
     "pt", "ru", "sv", "th", "tlh", "tr", "zh"
 ]
+# Words that are allowed since they are common subwords in languages without
+# spaces. These each filter >10% of documents of their language when disallowed.
+_BADWORDS_ALLOWLIST = {"ja": {"sm", "グロ", "女の子"}, "zh": {"性"}}
 
 DEFAULT_CC_VERSION = "2019-18"
 
@@ -98,41 +112,96 @@ OPENWEBTEXT_CC_VERSIONS = (  # August 2018 - July 2019
     "2018-39",
     "2018-34")
 
-ALL_CC_VERSIONS = (  # as of September 23, 2020
-    "2013-20", "2013-48", "2014-10", "2014-15", "2014-23", "2014-35", "2014-41",
-    "2014-42", "2014-49", "2014-52", "2015-06", "2015-11", "2015-14", "2015-18",
-    "2015-22", "2015-27", "2015-32", "2015-35", "2015-40", "2015-48", "2016-07",
-    "2016-18", "2016-22", "2016-26", "2016-30", "2016-36", "2016-40", "2016-44",
-    "2016-50", "2017-04", "2017-09", "2017-13", "2017-17", "2017-22", "2017-26",
-    "2017-30", "2017-34", "2017-39", "2017-43", "2017-47", "2017-51", "2018-05",
-    "2018-09", "2018-13", "2018-17", "2018-22", "2018-26", "2018-30", "2018-34",
-    "2018-39", "2018-43", "2018-47", "2018-51", "2019-04", "2019-09", "2019-13",
-    "2019-18", "2019-22", "2019-26", "2019-30", "2019-35", "2019-39", "2019-43",
-    "2019-47", "2019-51", "2020-05", "2020-10", "2020-16", "2020-24", "2020-29",
-    "2020-34")
+ALL_CC_VERSIONS = (  # as of August 1, 2022
+    "2013-20",
+    "2013-48",
+    "2014-10",
+    "2014-15",
+    "2014-23",
+    "2014-35",
+    "2014-41",
+    "2014-42",
+    "2014-49",
+    "2014-52",
+    "2015-06",
+    "2015-11",
+    "2015-14",
+    "2015-18",
+    "2015-22",
+    "2015-27",
+    "2015-32",
+    "2015-35",
+    "2015-40",
+    "2015-48",
+    "2016-07",
+    "2016-18",
+    "2016-22",
+    "2016-26",
+    "2016-30",
+    "2016-36",
+    "2016-40",
+    "2016-44",
+    "2016-50",
+    "2017-04",
+    "2017-09",
+    "2017-13",
+    "2017-17",
+    "2017-22",
+    "2017-26",
+    "2017-30",
+    "2017-34",
+    "2017-39",
+    "2017-43",
+    "2017-47",
+    "2017-51",
+    "2018-05",
+    "2018-09",
+    "2018-13",
+    "2018-17",
+    "2018-22",
+    "2018-26",
+    "2018-30",
+    "2018-34",
+    "2018-39",
+    "2018-43",
+    "2018-47",
+    "2018-51",
+    "2019-04",
+    "2019-09",
+    "2019-13",
+    "2019-18",
+    "2019-22",
+    "2019-26",
+    "2019-30",
+    "2019-35",
+    "2019-39",
+    "2019-43",
+    "2019-47",
+    "2019-51",
+    "2020-05",
+    "2020-10",
+    "2020-16",
+    "2020-24",
+    "2020-29",
+    "2020-34",
+    "2020-40",
+    "2020-45",
+    "2020-50",
+    "2021-04",
+    "2021-10",
+    "2021-17",
+    "2021-21",
+    "2021-25",
+    "2021-31",
+    "2021-39",
+    "2021-43",
+    "2021-49",
+    "2022-05",
+    "2022-21",
+    "2022-27",
+)
 
-_KNOWN_CORRUPT_WET_FILES = (  # as of September 23, 2020
-    # files that raise EOFError
-    "crawl-data/CC-MAIN-2016-50/segments/1480698543577.51/wet/CC-MAIN-20161202170903-00294-ip-10-31-129-80.ec2.internal.warc.wet.gz",
-    "crawl-data/CC-MAIN-2017-43/segments/1508187823309.55/wet/CC-MAIN-20171019141046-20171019161046-00789.warc.wet.gz",
-    "crawl-data/CC-MAIN-2017-47/segments/1510934805466.25/wet/CC-MAIN-20171119080836-20171119100836-00043.warc.wet.gz",
-    "crawl-data/CC-MAIN-2017-47/segments/1510934805466.25/wet/CC-MAIN-20171119080836-20171119100836-00043.warc.wet.gz",
-    "crawl-data/CC-MAIN-2017-47/segments/1510934805809.59/wet/CC-MAIN-20171119210640-20171119230640-00044.warc.wet.gz",
-    "crawl-data/CC-MAIN-2017-47/segments/1510934806543.24/wet/CC-MAIN-20171122084446-20171122104446-00120.warc.wet.gz",
-    "crawl-data/CC-MAIN-2017-51/segments/1512948517350.12/wet/CC-MAIN-20171212153808-20171212173808-00039.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-05/segments/1516084887660.30/wet/CC-MAIN-20180118230513-20180119010513-00778.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-09/segments/1518891815951.96/wet/CC-MAIN-20180224211727-20180224231727-00311.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-26/segments/1529267863518.39/wet/CC-MAIN-20180620104904-20180620124904-00402.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-26/segments/1529267863518.39/wet/CC-MAIN-20180620104904-20180620124904-00402.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-26/segments/1529267865995.86/wet/CC-MAIN-20180624005242-20180624025242-00197.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-30/segments/1531676591837.34/wet/CC-MAIN-20180720213434-20180720233434-00442.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-34/segments/1534221211167.1/wet/CC-MAIN-20180816191550-20180816211550-00078.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-34/segments/1534221211185.57/wet/CC-MAIN-20180816211126-20180816231126-00076.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-34/segments/1534221211185.57/wet/CC-MAIN-20180816211126-20180816231126-00076.warc.wet.gz",
-    "crawl-data/CC-MAIN-2018-34/segments/1534221219109.94/wet/CC-MAIN-20180821210655-20180821230655-00654.warc.wet.gz",
-    "crawl-data/CC-MAIN-2019-47/segments/1573496672170.93/wet/CC-MAIN-20191122222322-20191123011322-00000.warc.wet.gz",
-    "crawl-data/CC-MAIN-2020-24/segments/1590347458095.68/wet/CC-MAIN-20200604192256-20200604222256-00235.warc.wet.gz",
-    "crawl-data/CC-MAIN-2020-34/segments/1596439738819.78/wet/CC-MAIN-20200811180239-20200811210239-00123.warc.wet.gz",
+_KNOWN_CORRUPT_WET_FILES = (  # as of August 12, 2022
     # files that raise UnicodeDecodeError
     "crawl-data/CC-MAIN-2017-13/segments/1490218203536.73/wet/CC-MAIN-20170322213003-00052-ip-10-233-31-227.ec2.internal.warc.wet.gz",
 )
@@ -157,15 +226,15 @@ class C4Config(tfds.core.BuilderConfig):
   """BuilderConfig for C4 dataset."""
 
   def __init__(self,
-               name,
-               languages,
-               cc_versions=None,
-               clean=False,
-               badwords_filter=False,
-               paragraph_filter=False,
-               dedupe=True,
-               realnewslike=False,
-               webtextlike=False,
+               name: str,
+               languages: Sequence[str],
+               cc_versions: Optional[Sequence[str]] = None,
+               clean: bool = False,
+               badwords_filter_fraction: float = 0.0,
+               paragraph_filter: bool = False,
+               dedupe: bool = True,
+               realnewslike: bool = False,
+               webtextlike: bool = False,
                **kwargs):
     """BuilderConfig for C4.
 
@@ -176,8 +245,9 @@ class C4Config(tfds.core.BuilderConfig):
         use as the raw source text. Set to None to use default.
       clean: bool, whether to heuristically filter out lines and pages
         considered low quality. Note: only expected to work reliably for English
-          pages.
-      badwords_filter: bool, whether to filter out pages with badwords.
+        pages.
+      badwords_filter_fraction: float, what fraction of pages to filter out that
+        contain "bad words".
       paragraph_filter: bool, whether to filter out pages with too few or too
         short paragraphs.
       dedupe: bool, whether to deduplicate the dataset by paragraphs.
@@ -196,10 +266,14 @@ class C4Config(tfds.core.BuilderConfig):
       logging.warn(
           "C4 cleaning is only expected to work reliably for English pages.")
 
+    if not 0.0 <= badwords_filter_fraction <= 1.0:
+      raise ValueError(
+          "`badwords_filter_fraction` must be between 0.0 and 1.0.")
+
     self.languages = languages
     self.cc_versions = cc_versions or (DEFAULT_CC_VERSION,)
     self.clean = clean
-    self.badwords_filter = badwords_filter
+    self.badwords_filter_fraction = badwords_filter_fraction
     self.paragraph_filter = paragraph_filter
     self.dedupe = dedupe
     self.realnewslike = realnewslike
@@ -221,14 +295,14 @@ class C4(tfds.core.BeamBasedBuilder):
           languages=["en"],
           clean=True,
           dedupe=True,
-          badwords_filter=True,
+          badwords_filter_fraction=1.0,
           description="English C4 dataset."),
       C4Config(
           "en.noclean",
           languages=["en"],
           clean=False,
           dedupe=False,
-          badwords_filter=False,
+          badwords_filter_fraction=0.0,
           description="Disables all cleaning (deduplication, removal based on bad words, "
           "etc.)"),
       C4Config(
@@ -237,7 +311,7 @@ class C4(tfds.core.BeamBasedBuilder):
           realnewslike=True,
           clean=True,
           dedupe=True,
-          badwords_filter=True,
+          badwords_filter_fraction=1.0,
           description="Filters from the default config to only include content from the "
           "domains used in the 'RealNews' dataset (Zellers et al., 2019)."),
       C4Config(
@@ -247,7 +321,7 @@ class C4(tfds.core.BeamBasedBuilder):
           webtextlike=True,
           clean=True,
           dedupe=True,
-          badwords_filter=True,
+          badwords_filter_fraction=1.0,
           description="Filters from the default config to only include content from the "
           "URLs in OpenWebText (https://github.com/jcpeterson/openwebtext)."),
       C4Config(
@@ -257,8 +331,8 @@ class C4(tfds.core.BeamBasedBuilder):
           clean=False,
           paragraph_filter=True,
           dedupe=True,
-          badwords_filter=True,
-          description="Multilingual C4 (mC4) has 101 languages and is generated from 71 "
+          badwords_filter_fraction=0.999,
+          description="Multilingual C4 (mC4) has 101 languages and is generated from 86 "
           "Common Crawl dumps."),
   ]
 
@@ -279,8 +353,6 @@ class C4(tfds.core.BeamBasedBuilder):
     )
 
   def _split_generators(self, dl_manager, pipeline):
-    dl_manager.download_checksums(_CHECKSUMS_URL)
-
     # We will automatically download the first default CC version, but others
     # need to be manually downloaded.
     cc_versions = set(self.builder_config.cc_versions)
@@ -289,7 +361,7 @@ class C4(tfds.core.BeamBasedBuilder):
         _WET_PATH_URL.format(cc_version=cc_version)
         for cc_version in cc_versions
     ]
-    if self.builder_config.badwords_filter:
+    if self.builder_config.badwords_filter_fraction > 0.0:
       files_to_download["badwords"] = {
           lang: _BADWORDS_URL.format(lang=lang)
           for lang in _BADWORDS_LANGS
@@ -313,16 +385,12 @@ class C4(tfds.core.BeamBasedBuilder):
 
     file_paths = tf.nest.map_structure(os.fspath, file_paths)
 
-    page_content_pcollection = self._get_page_content(pipeline, file_paths,
-                                                      dl_manager)
+    pages_pcollection = self._get_pages_pcollection(pipeline, file_paths,
+                                                    dl_manager)
 
-    def _lang_filter(url_and_page, lang):
-      _, page = url_and_page
-      return page["language"] == lang
-
-    def _filter(url_and_page, lang, predicate_fn):
-      return (_lang_filter(url_and_page, lang) and
-              c4_utils.get_hashed_url_filter_fn(predicate_fn)(url_and_page))
+    def _filter(page, lang, predicate_fn):
+      return (page.language == lang and
+              c4_utils.get_hashed_url_filter_fn(predicate_fn)(page))
 
     train_predicate_fn = lambda x: x % 1000 != 0  # 99.9%
     validation_predicate_fn = lambda x: x % 1000 == 0  # 00.1%
@@ -334,7 +402,7 @@ class C4(tfds.core.BeamBasedBuilder):
               name=tfds.Split.TRAIN,
               gen_kwargs=dict(
                   split="train",
-                  page_content=page_content_pcollection,
+                  pages=pages_pcollection,
                   split_filter_fn=c4_utils.get_hashed_url_filter_fn(
                       predicate_fn=train_predicate_fn)),
           ),
@@ -342,7 +410,7 @@ class C4(tfds.core.BeamBasedBuilder):
               name=tfds.Split.VALIDATION,
               gen_kwargs=dict(
                   split="validation",
-                  page_content=page_content_pcollection,
+                  pages=pages_pcollection,
                   split_filter_fn=c4_utils.get_hashed_url_filter_fn(
                       predicate_fn=validation_predicate_fn)),
           ),
@@ -355,7 +423,7 @@ class C4(tfds.core.BeamBasedBuilder):
               name=lang,
               gen_kwargs=dict(
                   split=lang,
-                  page_content=page_content_pcollection,
+                  pages=pages_pcollection,
                   split_filter_fn=functools.partial(
                       _filter, lang=lang, predicate_fn=train_predicate_fn),
               )),
@@ -363,14 +431,14 @@ class C4(tfds.core.BeamBasedBuilder):
               name=f"{lang}-validation",
               gen_kwargs=dict(
                   split=f"{lang}-validation",
-                  page_content=page_content_pcollection,
+                  pages=pages_pcollection,
                   split_filter_fn=functools.partial(
                       _filter, lang=lang, predicate_fn=validation_predicate_fn),
               ))
       ])
     return splits
 
-  def _get_page_content(self, pipeline, file_paths, dl_manager):
+  def _get_pages_pcollection(self, pipeline, file_paths, dl_manager):
     """Build PCollection of un-split page content."""
     beam = tfds.core.lazy_imports.apache_beam
 
@@ -385,7 +453,7 @@ class C4(tfds.core.BeamBasedBuilder):
       tmp_dir = f"{out_path}.incomplete{uuid.uuid4().hex}"
       try:
         tf.io.gfile.makedirs(tmp_dir)
-        downloader = tfds.download.downloader.get_downloader()
+        downloader = tfds.download.download_manager.get_downloader()
         with downloader.tqdm():
           # TODO(slebedev): Investigate why pytype infers Promise[Future[...]].
           dl_path = downloader.download(url, tmp_dir).get().path  # type: ignore
@@ -411,71 +479,68 @@ class C4(tfds.core.BeamBasedBuilder):
             dl_dir=os.path.join(dl_manager.download_dir, "c4_wet_files")))
 
     # Parse WET files and filter by length.
-    # Output: url, text
-    page_content = (
+    # Output: [PageFeatures]
+    pages = (
         wet_file_paths
         | beam.FlatMap(c4_utils.split_wet_file)
         | beam.Filter(c4_utils.is_valid_length))
 
     # Optionally filter for RealNews domains.
-    # Output: url, text
+    # Output: [PageFeatures]
     if self.builder_config.realnewslike:
       with tf.io.gfile.GFile(file_paths["realnews_domains"]) as f:
         realnews_domains = json.load(f)
-      page_content = (
-          page_content
-          | beam.Filter(c4_utils.is_realnews_domain, realnews_domains))
+      pages |= beam.Filter(c4_utils.is_realnews_domain, realnews_domains)
 
     # Normalize and deduplicate by URL.
-    # Output: url, text
-    page_content = (
-        page_content
-        | "normalize_url" >> beam.Map(c4_utils.normalize_url)
+    # Output: [PageFeatures]
+    pages = (
+        pages
+        | "normalize_url" >> beam.Map(lambda p: (p.normalized_url, p))
         | "group_url" >> beam.GroupByKey()
-        | beam.Map(c4_utils.dedupe_urls))
+        | beam.Values()
+        | beam.Map(c4_utils.select_newest_page))
 
     # Optionally filter for WebText-like URLs.
-    # Output: url, text
+    # Output: [PageFeatures]
     if self.builder_config.webtextlike:
       webtextlike_urls = (
           pipeline
           | "read_webtextlike_urls" >> beam.io.ReadFromText(
               os.path.join(file_paths["openwebtext_urls_zip"],
                            _OPENWEBTEXT_URLS_FILE_PATTERN))
-          | "add_dummy_page" >> beam.Map(lambda x: (x, ""))
-          | "normal_webtext_url" >> beam.Map(c4_utils.normalize_url))
-      page_content = ({
-          "text": page_content,
+          | "add_dummy_page" >>
+          beam.Map(lambda url: (c4_utils.normalize_url(url), PageFeatures())))
+      pages = ({
+          "pages": pages | beam.Map(lambda p: (p.normalized_url, p)),
           "webtextlike_urls": webtextlike_urls
       }
-                      | "group_webtextlike_urls" >> beam.CoGroupByKey()
-                      | beam.FlatMap(c4_utils.filter_by_webtextlike))
+               | "group_webtextlike_urls" >> beam.CoGroupByKey()
+               | beam.FlatMap(c4_utils.filter_by_webtextlike))
 
     if self.builder_config.paragraph_filter:
-      page_content |= beam.Filter(c4_utils.paragraph_filter)
+      pages |= beam.Filter(c4_utils.paragraph_filter)
 
     if self.builder_config.clean:
-      page_content = (
-          page_content
-          | "clean_pages" >> beam.FlatMap(c4_utils.get_clean_page_fn()))
+      pages |= ("clean_pages" >> beam.FlatMap(c4_utils.get_clean_page_fn()))
 
     if self.builder_config.dedupe:
-      page_content = (
+      pages = (
           # Also removes documents with too few sentences after deduplication.
-          c4_utils.remove_duplicate_text(page_content)  # pylint:disable=g-long-ternary
+          c4_utils.remove_duplicate_text(pages)  # pylint:disable=g-long-ternary
           if self.builder_config.clean else
           # If we are not cleaning, do not remove too-few-sentence documents.
-          c4_utils.remove_duplicate_text(page_content, min_num_sentences=0))
+          c4_utils.remove_duplicate_text(pages, min_num_sentences=0))
 
     # Add detected language.
     if self.builder_config.languages == ["en"]:
       # Use langdetect for reproducibility of the original C4.
-      page_content |= beam.FlatMap(c4_utils.detect_english)
+      pages |= beam.FlatMap(c4_utils.detect_english)
     else:
-      page_content = c4_utils.detect_languages(
-          page_content, valid_languages=self.builder_config.languages)
+      pages = c4_utils.detect_languages(
+          pages, valid_languages=self.builder_config.languages)
 
-    if self.builder_config.badwords_filter:
+    if self.builder_config.badwords_filter_fraction > 0.0:
       # Create dictionary of badwords regex for each available language.
       badwords = collections.defaultdict(set)
       for lang, path in file_paths["badwords"].items():
@@ -483,25 +548,27 @@ class C4(tfds.core.BeamBasedBuilder):
         with tf.io.gfile.GFile(path) as f:
           badwords[lang].update(l.strip() for l in f)
 
-      page_content |= beam.Filter(c4_utils.get_badwords_filter_fn(badwords))
+      for lang, allowlist in _BADWORDS_ALLOWLIST.items():
+        badwords[lang] -= allowlist
 
-    return page_content
+      pages |= beam.Filter(
+          c4_utils.get_badwords_filter_fn(
+              badwords,
+              filter_fraction=self.builder_config.badwords_filter_fraction))
 
-  def _build_pcollection(self, unused_pipeline, split, page_content,
-                         split_filter_fn):
+    return pages
+
+  def _build_pcollection(self, unused_pipeline, split, pages, split_filter_fn):
     beam = tfds.core.lazy_imports.apache_beam
 
-    def _emit_examples(el):
+    def _emit_examples(page: PageFeatures):
       c4_utils.get_counter_inc_fn(split)("examples")
-      _, features = el
-      return features["url"], {
-          "url": features["url"],
-          "text": features["text"],
-          "content-type": features["content-type"],
-          "content-length": features["content-length"],
-          "timestamp": features["timestamp"]
+      return page.url, {
+          "url": page.url,
+          "text": page.text,
+          "content-type": page.content_type,
+          "content-length": page.content_length,
+          "timestamp": page.timestamp
       }
 
-    return (page_content
-            | beam.Filter(split_filter_fn)
-            | beam.Map(_emit_examples))
+    return (pages | beam.Filter(split_filter_fn) | beam.Map(_emit_examples))
