@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 
 """Utilities for generating the C4 dataset."""
 
+import dataclasses
 import functools
 import gzip
 import hashlib
@@ -22,9 +23,10 @@ import heapq
 import io
 import re
 import threading
+from typing import Collection, Iterable, Mapping, Optional, Sequence, Tuple
 
 from absl import logging
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 import tensorflow_datasets.public_api as tfds
 
 # WET file constants
@@ -52,6 +54,17 @@ _SENTENCE_TOKENIZER = None
 UNKNOWN_LANGUAGE = "und"
 
 
+@dataclasses.dataclass
+class PageFeatures:
+  url: str = ""
+  normalized_url: str = ""
+  text: str = ""
+  timestamp: str = ""
+  content_length: str = ""
+  content_type: str = ""
+  language: Optional[str] = None
+
+
 def get_counter_inc_fn(namespace):
 
   def counter_inc_fn(counter, amt=1):
@@ -63,8 +76,8 @@ def get_counter_inc_fn(namespace):
 
 def get_hashed_url_filter_fn(predicate_fn):
 
-  def filter_fn(el):
-    url, _ = el
+  def filter_fn(page):
+    url = page.normalized_url
     val = int(
         hashlib.md5(tf.compat.as_text(url).encode("utf-8")).hexdigest(), 16)
     return predicate_fn(val)
@@ -97,10 +110,9 @@ def _get_sentences(text):
 langdetect_lock = threading.Lock()
 
 
-def detect_english(page, min_probability=0.99):
+def detect_english(page: PageFeatures, min_probability=0.99):
   """Yields page iff text is 'en' with at least `min_probability`."""
-  url, features = page
-  text = features["text"]
+  text = page.text
 
   counter_inc_fn = get_counter_inc_fn("english-filter")
 
@@ -126,9 +138,7 @@ def detect_english(page, min_probability=0.99):
     return
 
   counter_inc_fn("passed")
-  features = dict(features)
-  features["language"] = "en"
-  yield url, features
+  yield dataclasses.replace(page, language="en")
 
 
 def detect_languages(pages, valid_languages):
@@ -139,7 +149,7 @@ def detect_languages(pages, valid_languages):
   class _PredictLanguageFn(beam.DoFn):
     """Predicts page's language using cld3 and adds to features."""
 
-    def __init__(self, valid_languages, min_probability=0.7):
+    def __init__(self, valid_languages, min_probability=0.95):
       self._valid_languages = set(valid_languages)
       self._counter_inc_fn = get_counter_inc_fn("language-filter")
       self._min_probability = min_probability
@@ -152,11 +162,9 @@ def detect_languages(pages, valid_languages):
             min_num_bytes=100,
             max_num_bytes=10000)
 
-    def process(self, page):
-      url, features = page
-      features = dict(features)
+    def process(self, page: PageFeatures):
       with langdetect_lock:
-        result = self._detector.FindLanguage(features["text"])
+        result = self._detector.FindLanguage(page.text)
       if not result.is_reliable:
         self._counter_inc_fn("filtered:no_predictions")
         lang = UNKNOWN_LANGUAGE
@@ -170,8 +178,7 @@ def detect_languages(pages, valid_languages):
           return
       self._counter_inc_fn("passed")
       self._counter_inc_fn("passed:%s" % lang)
-      features["language"] = lang
-      yield url, features
+      yield dataclasses.replace(page, language=lang)
 
   return pages | beam.ParDo(_PredictLanguageFn(valid_languages=valid_languages))
 
@@ -183,12 +190,12 @@ def get_clean_page_fn():
   return functools.partial(clean_page, citation_regex=citation_regex)
 
 
-def clean_page(url_and_features,
+def clean_page(page: PageFeatures,
                citation_regex,
                counter_inc_fn=None,
                min_words_per_line=_MIN_WORDS_PER_LINE,
                min_num_sentences=_MIN_NUM_SENTENCES,
-               max_word_length=_MAX_WORD_LENGTH):
+               max_word_length=_MAX_WORD_LENGTH) -> Iterable[PageFeatures]:
   """Cleans a CommonCrawl page, yielding nothing if it should be skipped.
 
   Cleaning removes lines with no end marks or with too few words. After line
@@ -196,7 +203,7 @@ def clean_page(url_and_features,
   simple count of end marks.
 
   Args:
-    url_and_features: tuple(string, dict), the url and features of the page.
+    page: the features of the page.
     citation_regex: Regex to use for finding Wikipedia-like citations to filter.
     counter_inc_fn: function, a function taking the name of a counter to be
       incremented and the (optional) amount. Defaults to a beam Metric counter.
@@ -210,8 +217,7 @@ def clean_page(url_and_features,
   Yields:
     The url and cleaned text for the page.
   """
-  url, features = url_and_features
-  text = features["text"]
+  text = page.text
 
   if not counter_inc_fn:
     counter_inc_fn = get_counter_inc_fn("clean-page")
@@ -263,23 +269,22 @@ def clean_page(url_and_features,
     counter_inc_fn("filtered:too_few_sentences")
     return
   counter_inc_fn("passed")
-  features["text"] = "\n".join(valid_lines).strip()
-  yield url, features
+  yield dataclasses.replace(page, text="\n".join(valid_lines).strip())
 
 
 def _hash_text(text):
   return hashlib.md5(tf.compat.as_text(text).encode("utf-8")).hexdigest()
 
 
-def _emit_url_to_lines(page):
+def _emit_url_to_lines(page: PageFeatures) -> Iterable[Tuple[str, str]]:
   """Emits url to all (lower-cased, hashed) lines."""
-  url, features = page
-  text = features["text"]
+  text = page.text
   for line in text.split("\n"):
-    yield _hash_text(line.strip().lower()), url
+    yield _hash_text(line.strip().lower()), page.url
 
 
-def _remove_lines_from_text(el, counter_inc_fn, min_num_sentences):
+def _remove_lines_from_text(el, counter_inc_fn,
+                            min_num_sentences) -> Iterable[PageFeatures]:
   """Removes all lines from the page that do not match the given set of hashes.
 
   Process the result of a join containing a single value for 'features' and zero
@@ -287,7 +292,7 @@ def _remove_lines_from_text(el, counter_inc_fn, min_num_sentences):
   line that has been selected to keep.
 
   Args:
-    el: `(string, {'features': features_dict, 'lines': [string]})`, element
+    el: `(string, {'pages': PageFeatures, 'lines': [string]})`, element
       containing the result of a join on key with both the page text and
       lower-cased, hashed lines to remove.
     counter_inc_fn: function, a function taking the name of a counter to be
@@ -296,16 +301,14 @@ def _remove_lines_from_text(el, counter_inc_fn, min_num_sentences):
       be skipped.
 
   Yields:
-    url: The URL of the page.
-    features: The page features with lines removed from text.
+    The page features with lines removed from text.
   """
   url, join_values = el
-  features = join_values["features"]
+  pages = join_values["pages"]
 
-  assert len(features) == 1, "Invalid page count (%d) for %s" % (len(features),
-                                                                 url)
-  features = features[0]
-  text = features["text"]
+  assert len(pages) == 1, "Invalid page count (%d) for %s" % (len(pages), url)
+  page = pages[0]
+  text = page.text
   lines_to_keep = set(join_values["lines"])
   new_lines = []
   hashed_lines = set()
@@ -327,14 +330,11 @@ def _remove_lines_from_text(el, counter_inc_fn, min_num_sentences):
     counter_inc_fn("filtered:too_few_sentences")
     return
   counter_inc_fn("passed")
-  new_features = features.copy()
-  new_features["text"] = new_text
-  yield (url, new_features)
+  yield dataclasses.replace(page, text=new_text)
 
 
 def remove_duplicate_text(pages, min_num_sentences=_MIN_NUM_SENTENCES):
   """Utility to remove duplicate lines across text documents."""
-  # Output: url, lines
   beam = tfds.core.lazy_imports.apache_beam
 
   # Select a single URL for each line in the input pages.
@@ -349,7 +349,7 @@ def remove_duplicate_text(pages, min_num_sentences=_MIN_NUM_SENTENCES):
 
   # Output: url, text
   final_docs = ({
-      "features": pages,
+      "pages": pages | beam.Map(lambda p: (p.url, p)),
       "lines": lines_to_keep
   }
                 | "group_features_and_lines_by_url" >> beam.CoGroupByKey()
@@ -368,106 +368,85 @@ def split_wet_file(wet_file_path, counter_inc_fn=None):
     counter_inc_fn = get_counter_inc_fn("split-wet-file")
   counter_inc_fn("wet-file")
 
+  def _validate_features(page):
+    """Return True if page features are valid."""
+    if not page.url:
+      counter_inc_fn("filtered:no_url")
+    elif not page.text:
+      counter_inc_fn("filtered:no_content")
+    elif not page.timestamp:
+      counter_inc_fn("filtered:no_timestamp")
+    else:
+      counter_inc_fn("passed")
+      return True
+    return False
+
   with tf.io.gfile.GFile(wet_file_path,
                          "rb") as f, gzip.GzipFile(fileobj=f) as g:
-    url = None
-    content = None
-    content_len = None
-    content_type = None
-    timestamp = None
-
-    def _maybe_get_page():
-      """Generate a (url, {features}) page."""
-      if not url and url is not None:
-        counter_inc_fn("filtered:no_url")
-      if not content and content is not None:
-        counter_inc_fn("filtered:no_content")
-      if not content_type and content_type is not None:
-        counter_inc_fn("filtered:no_content_type")
-      if not content_len and content_len is not None:
-        counter_inc_fn("filtered:no_content_len")
-      if not timestamp and timestamp is not None:
-        counter_inc_fn("filtered:no_timestamp")
-      if content and url:
-        counter_inc_fn("passed")
-        return (url, {
-            "text": "\n".join(content),
-            "content-type": content_type,
-            "content-length": content_len,
-            "timestamp": timestamp,
-            "url": url
-        })
-      return None
-
-    for line in io.TextIOWrapper(g, encoding="utf-8"):  # pytype: disable=wrong-arg-types
+    page = PageFeatures()
+    for i, line in enumerate(io.TextIOWrapper(g, encoding="utf-8")):  # pytype: disable=wrong-arg-types
       line = line.strip()
       if not line:
         continue
       if line == _PAGE_DELIMITER:
-        page = _maybe_get_page()
-        if page:
+        if i > 0 and _validate_features(page):
           yield page
-        url = ""
-        content = []
-        content_len = ""
-        content_type = ""
-        timestamp = ""
+        page = PageFeatures()
 
       if line.startswith(_URL_KEY):
-        url = line[len(_URL_KEY):].strip()
+        page.url = line[len(_URL_KEY):].strip()
+        page.normalized_url = normalize_url(line[len(_URL_KEY):].strip())
 
       if line.startswith(_URL_DATE):
-        timestamp = line[len(_URL_DATE):].strip()
+        page.timestamp = line[len(_URL_DATE):].strip()
 
       if line.startswith(_CONTENT_TYPE):
-        content_type = line[len(_CONTENT_TYPE):].strip()
+        page.content_type = line[len(_CONTENT_TYPE):].strip()
 
       if line.startswith(_CONTENT_LEN):
-        content_len = line[len(_CONTENT_LEN):].strip()
+        page.content_length = line[len(_CONTENT_LEN):].strip()
 
       if line.startswith(_METADATA_PREFIXES):
         continue
 
-      content.append(line)  # pytype: disable=attribute-error
+      if page.text:
+        page.text += "\n"
+      page.text += line
 
-    page = _maybe_get_page()
-    if page:
+    if _validate_features(page):
       yield page
 
 
-def dedupe_urls(el):
-  """Deterministically return as random page for a given URL."""
+def select_newest_page(pages):
+  """Deterministically return as random page."""
   counter_inc_fn = get_counter_inc_fn("duplicate-url-filter")
 
-  url, pages = el
   cnt = 0
-  page, page_hash = None, None
+  selected_page = None
   for p in pages:
     cnt += 1
-    p_hash = _hash_text(p["text"])
-    if not page_hash or p_hash > page_hash:
-      page = p
-      page_hash = p_hash
+    if not selected_page or p.timestamp > selected_page.timestamp:
+      selected_page = p
   counter_inc_fn("filtered", cnt - 1)
   counter_inc_fn("passed")
-  return url, page
+  return selected_page
 
 
-def is_valid_length(el, max_length=1.9e5):
+def is_valid_length(page: PageFeatures, max_length=1.9e5):
   """Returns False iff page's text is too long."""
   counter_inc_fn = get_counter_inc_fn("too-long-filter")
-  _, page = el
-  if len(page["text"]) > max_length:
+  if len(page.text) > max_length:
     counter_inc_fn("filtered")
     return False
   counter_inc_fn("passed")
   return True
 
 
-def is_realnews_domain(el, realnews_domains):
+def is_realnews_domain(page: PageFeatures,
+                       realnews_domains: Mapping[str, Collection[str]]):
   """Returns False iff page's (sub)domain is not allowed."""
   counter_inc_fn = get_counter_inc_fn("realnews-domain-filter")
-  url, _ = el
+  url = page.normalized_url
   ext = tfds.core.lazy_imports.tldextract.extract(url)
   main_domain = ext.domain + "." + ext.suffix
   if main_domain not in realnews_domains:
@@ -485,48 +464,55 @@ def is_realnews_domain(el, realnews_domains):
 def filter_by_webtextlike(el):
   """Yields only pages with a matching WebText-like URL."""
   counter_inc_fn = get_counter_inc_fn("webtextlike-filter")
-  url, join_values = el
-  text = join_values["text"]
+  _, join_values = el
+  pages = join_values["pages"]
   webtextlike = join_values["webtextlike_urls"]
   if not webtextlike:
     counter_inc_fn("filtered")
     return
-  if not text:
+  if not pages:
     counter_inc_fn("missing-page")
     return
-  assert len(text) == 1
+  assert len(pages) == 1
   counter_inc_fn("passed")
-  yield url, text[0]
+  yield pages[0]
 
 
-def normalize_url(el):
-  url, val = el
+def normalize_url(url):
   url = tf.compat.as_text(url)
   url = re.sub(r"https?:\/\/(www\.)?", "", url)
   url = re.sub(r"\?(utm_|ref|feed).*", "", url)
   url = url.rstrip("/")
-  return url, val
+  return url
 
 
-def get_badwords_filter_fn(badwords):
-  """Filters pages that contain any language-specific bad words."""
-  badwords_regex = {  # pylint:disable=g-complex-comprehension
-      lang: (
-          # For Chinese and Thai, match bad words regardless of context.
-          re.compile("|".join(words)) if lang in ("th", "zh")
-          # For other languages, match only when flanked by non-word chars.
-          else re.compile(r"(?:\W|^)({})(?:\W|$)".format("|".join(words))))
-      for lang, words in badwords.items()
-  }
+def get_badwords_filter_fn(badwords: Mapping[str, Sequence[str]],
+                           filter_fraction: float = 1.0):
+  """Filters pages at given rate that contain language-specific bad word(s)."""
+  badwords_regex = {}
+  for lang, words in badwords.items():
+    words = [re.escape(w) for w in words]
+    badwords_regex[lang] = (
+        # For Japanese, Thai, and Chinese, do not require word separations.
+        re.compile("|".join(words)) if lang in ("ja", "th", "zh")
+        # For other languages, match only when flanked by non-word chars.
+        else re.compile(r"(?:\W|^)({})(?:\W|$)".format("|".join(words))))
+
+  filter_ratio = float.as_integer_ratio(filter_fraction)
+  keep_badword_page = get_hashed_url_filter_fn(
+      lambda x: x % filter_ratio[1] >= filter_ratio[0])
 
   def badwords_filter(page):
-    _, features = page
-    lang = features["language"].split("-")[0]  # remove suffix if present
+    lang = page.language.split("-")[0]  # remove suffix if present
 
     if lang in badwords_regex:
-      text = features["text"]
+      text = page.text
       badwords_found = badwords_regex[lang].search(text.lower())
       if badwords_found is not None:
+        if keep_badword_page(page):
+          get_counter_inc_fn("badwords-filter")("soft-passed")
+          get_counter_inc_fn("badwords-filter-%s" % lang)("soft-passed")
+          return True
         get_counter_inc_fn("badwords-filter")("filtered")
         get_counter_inc_fn("badwords-filter-%s" % lang)("filtered")
         return False
@@ -540,10 +526,7 @@ def get_badwords_filter_fn(badwords):
 
 def paragraph_filter(page, min_paragraphs=3, min_paragraph_len=200):
   """Returns False iff a page has too few or too short paragraphs."""
-
-  _, features = page
-
-  lines = features["text"].split("\n")
+  lines = page.text.split("\n")
   # Filter out docs that don't have at least three "paragraphs"
   # (lines >= `min_paragraph_len` chars).
   if (len(lines) < min_paragraphs or
